import { BlockLocation, Location, system, world } from "@minecraft/server"
import { ActionFormData } from "@minecraft/server-ui"

let drillBlocks = []

const dimensions = ['overworld', 'nether', 'the_end']

const nonBreakableBlocks = [
    'minecraft:air',
    'minecraft:bedrock',
    'minecraft:end_portal_frame',
    'minecraft:lava',
    'minecraft:water',
    'minecraft:flowing_lava',
    'minecraft:flowing_water',
    'effect99:quarry_drill',
    'effect99:steam_engine',
    'effect99:steam_generator',
    'effect99:quarry_structure',
    'effect99:medium_quarry_structure',
    'effect99:largue_quarry_structure'
]

world.events.beforeItemUseOn.subscribe(i => {
    const { x, y, z } = i.blockLocation
    const block = world.getDimension(i.source.dimension.id).getBlock(new BlockLocation(x, y, z))
    const player = i.source

    if (!player.hasTag("Form") && (block.typeId.includes('steam_generator') || block.typeId.includes('steam_engine')) && block.hasTag('machine_panel')) {
        player.addTag("Form")
        const panel = new ActionFormData()
            .title(`${block.typeId.slice(block.typeId.indexOf(":") + 1)[0].toUpperCase() + block.typeId.slice(block.typeId.indexOf(":") + 2).replace("_", " ").replace("_", " ")}`)
            .body('Dismantle Machine?').button('Yes').button('No')

        panel.show(player).then(r => {
            if (r.selection === 0) {
                if (block.hasTag('north_block')) {
                    player.runCommandAsync(`fill ${x - 1} ${y - 1} ${z - 2} ${x + 1} ${y + 1} ${z} air 0`)
                }
                else if (block.hasTag('south_block')) {
                    player.runCommandAsync(`fill ${x + 1} ${y + 1} ${z + 2} ${x - 1} ${y - 1} ${z} air 0`)
                }
                else if (block.hasTag('west_block')) {
                    player.runCommandAsync(`fill ${x - 2} ${y - 1} ${z - 1} ${x} ${y + 1} ${z + 1} air 0`)
                }
                else if (block.hasTag('east_block')) {
                    player.runCommandAsync(`fill ${x + 2} ${y + 1} ${z + 1} ${x} ${y - 1} ${z - 1} air 0`)
                }
                if (block.typeId.includes('steam_engine')) {
                    player.runCommandAsync(`execute @s ~~~ loot spawn ${x} ${y} ${z} loot "blocks/steam_engine"`)
                }
                else if (block.typeId.includes('steam_generator')) {
                    player.runCommandAsync(`execute @s ~~~ loot spawn ${x} ${y} ${z} loot "blocks/steam_generator"`)
                }
            }
            player.removeTag("Form")
        })
    }
})

world.events.tick.subscribe(t => {
    if (t.currentTick % 40 === 0 && drillBlocks[0]) {
        for (const { coords: { x, y, z }, rotation: r } of drillBlocks) {
            for (const worlds of dimensions) {
                try {
                    if (world.getDimension(worlds).getBlock(new BlockLocation(x, y, z)).typeId === 'effect99:quarry_drill') {
                        if (world.getDimension(worlds).getBlock(new BlockLocation(x, y, z)).permutation.getProperty('block:energy').value !== 0) {
                            if (r === 0 && !nonBreakableBlocks.includes(world.getDimension(worlds).getBlock(new BlockLocation(x, y + 1, z)).typeId)) {
                                blockPermutation(x, y + 1, z)
                            }
                            else if (r === 1 && !nonBreakableBlocks.includes(world.getDimension(worlds).getBlock(new BlockLocation(x, y - 1, z)).typeId)) {
                                blockPermutation(x, y - 1, z)
                            }
                            else if (r === 2 && !nonBreakableBlocks.includes(world.getDimension(worlds).getBlock(new BlockLocation(x, y, z + 1)).typeId)) {
                                blockPermutation(x, y, z + 1)
                            }
                            else if (r === 3 && !nonBreakableBlocks.includes(world.getDimension(worlds).getBlock(new BlockLocation(x, y, z - 1)).typeId)) {
                                blockPermutation(x, y, z - 1)
                            }
                            else if (r === 4 && !nonBreakableBlocks.includes(world.getDimension(worlds).getBlock(new BlockLocation(x + 1, y, z)).typeId)) {
                                blockPermutation(x + 1, y, z)
                            }
                            else if (r === 5 && !nonBreakableBlocks.includes(world.getDimension(worlds).getBlock(new BlockLocation(x - 1, y, z)).typeId)) {
                                blockPermutation(x - 1, y, z)
                            }
                            function blockPermutation(cx, cy, cz) {
                                world.getDimension(worlds).runCommandAsync(`setblock ${cx} ${cy} ${cz} air 0 destroy`)
                                const permutation = world.getDimension(worlds).getBlock(new BlockLocation(x, y, z)).permutation
                                permutation.getProperty('block:energy').value -= 1
                                world.getDimension(worlds).getBlock(new BlockLocation(x, y, z)).setPermutation(permutation)
                            }
                        }
                    }
                    else {
                        for (const detectPlayer of world.getDimension(worlds).getPlayers({ location: new Location(x, y, z), maxDistance: 80 })) {
                            drillBlocks.splice(drillBlocks.findIndex(f => (f.coords.x === x && f.coords.y === y && f.coords.z === z)), 1)
                            for (const player of world.getPlayers()) {
                                player.removeTag(`${player.getTags().find(f => { return f.includes(`{"coords":{"x":${x},"y":${y},"z":${z}}`) })}`)
                            }
                        }
                    }
                } catch (e) { }
            }
        }
    }
    if (t.currentTick % 10 === 0) {
        for (const player of world.getPlayers()) {
            try {
                const block = player.getBlockFromViewVector({ maxDistance: 8 })
                if (block.typeId === 'effect99:quarry_drill') {
                    if (drillBlocks.every(({ coords: f }) => (f.x !== block.x || f.y !== block.y || f.z !== block.z))) {
                        drillBlocks.unshift({ coords: { x: block.x, y: block.y, z: block.z }, rotation: block.permutation.getProperty('block:rotation').value })
                        for (const player of world.getPlayers()) {
                            try {
                                if (player.getTags().every(e => (JSON.parse([e].find(f => { return f.includes('{"coords":{"x":') })).coords.x !== block.x || JSON.parse([e].find(f => { return f.includes('{"coords":{"x":') })).coords.y !== block.y || JSON.parse([e].find(f => { return f.includes('{"coords":{"x":') })).coords.z !== block.z))) {
                                    player.addTag(`${JSON.stringify(drillBlocks[0])}`)
                                }
                            } catch (e) {
                                player.addTag(`${JSON.stringify(drillBlocks[0])}`)
                            }
                        }
                    }
                }
            } catch (e) { }
        }
    }
})

world.events.blockPlace.subscribe(b => {
    if (b.block.typeId === 'effect99:quarry_drill') {
        if (drillBlocks.every(({ coords: f }) => (f.x !== b.block.x || f.y !== b.block.y || f.z !== b.block.z))) {
            drillBlocks.unshift({ coords: { x: b.block.x, y: b.block.y, z: b.block.z }, rotation: b.block.permutation.getProperty('block:rotation').value })
            for (const player of world.getPlayers()) {
                try {
                    if (player.getTags().every(e => (JSON.parse([e].find(f => { return f.includes('{"coords":{"x":') })).coords.x !== b.block.x || JSON.parse([e].find(f => { return f.includes('{"coords":{"x":') })).coords.y !== b.block.y || JSON.parse([e].find(f => { return f.includes('{"coords":{"x":') })).coords.z !== b.block.z))) {
                        player.addTag(`${JSON.stringify(drillBlocks[0])}`)
                    }
                } catch (e) {
                    player.addTag(`${JSON.stringify(drillBlocks[0])}`)
                }
            }
        }
    }
})
world.events.blockBreak.subscribe(b => {
    if (b.brokenBlockPermutation.type.id === 'effect99:quarry_drill') {
        for (const player of world.getPlayers()) {
            drillBlocks.splice(drillBlocks.findIndex(f => (f.coords.x === b.block.x && f.coords.y === b.block.y && f.coords.z === b.block.z)), 1)
            player.removeTag(`{"coords":{"x":${b.block.x},"y":${b.block.y},"z":${b.block.z}},"rotation":${b.brokenBlockPermutation.getProperty('block:rotation').value}}`)
        }
    }
})

world.events.playerJoin.subscribe(p => {
    const joinedP = p.player.name
    const ticks = (t = 1) => system.run(() => {
        try {
            if (Math.floor(Math.random() * 10) % 4 === 0) {
                for (const player of world.getPlayers()) {
                    if (joinedP === player.name) {
                        if (player.getTags().some(e => e.includes('{"coords":{"x":'))) {
                            for (const tag of player.getTags()) {
                                if (tag.includes('{"coords":{"x":')) {
                                    if (drillBlocks.every(({ coords: f }) => (f.x !== JSON.parse(tag).coords.x || f.y !== JSON.parse(tag).coords.y || f.z !== JSON.parse(tag).coords.z))) {
                                        drillBlocks.push(JSON.parse(tag))
                                    }
                                }
                            }
                        }
                        player.removeTag('Form')
                        t = 0
                    }
                }
            }
        } catch (e) { }
        t ? ticks() : 0
    })
    ticks()
})

world.events.worldInitialize.subscribe(() => {
    const ticks = (t) => system.run(() => {
        try {
            if (t % 4 === 0) {
                for (const player of world.getPlayers()) {
                    player.removeTag('Form')
                    if (player.getTags().some(e => e.includes('{"coords":{"x":'))) {
                        for (const tag of player.getTags()) {
                            if (tag.includes('{"coords":{"x":')) {
                                if (drillBlocks.every(({ coords: f }) => (f.x !== JSON.parse(tag).coords.x || f.y !== JSON.parse(tag).coords.y || f.z !== JSON.parse(tag).coords.z))) {
                                    drillBlocks.push(JSON.parse(tag))
                                }
                            }
                        }
                    }
                    t = 0
                }
            }
        } catch (e) { }
        if (t) ticks(t - 1)
    })
    world.say('World Initialize')
    ticks(300)
})

world.events.beforeChat.subscribe(chat => {
    if (chat.message === '!rTags') {
        chat.cancel = true
        for (const player of world.getPlayers()) {
            for (let i = 0; player.getTags()[0]; i++) {
                player.removeTag(`${player.getTags()[0]}`)
            }
        }
    }
    else if (chat.message === '!dBlocks') {
        chat.cancel = true
        world.say(`${drillBlocks[0].rotation} - ${drillBlocks.length}`)
    }
})

const non_pushable_blocks = [
    'effect99:mechanical_arm',
    'effect99:fast_mechanical_arm',
    'effect99:regular_mechanical_arm',
    'effect99:antenna_base',
    'effect99:antenna',
    'effect99:antenna_signal',
    'effect99:steam_engine',
    'effect99:steam_generator',
    'effect99:quarry_structure',
    'effect99:medium_quarry_structure',
    'effect99:largue_quarry_structure',

    'effect99:quarry_drill' //Temporarily
]

world.events.beforePistonActivate.subscribe(p => {
    for (const { x, y, z } of p.piston.attachedBlocks) {
        if (non_pushable_blocks.includes(world.getDimension(p.dimension.id).getBlock(new BlockLocation(x, y, z)).typeId)) {
            p.cancel = true
        }
    }
})

//Add-On Created by Effect99